==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/server/inc/gw/gwapi.php <==
<?
//TODO : close access and delete session data 
//       on relogin


class MerakGWAPI {

	// PUBLIC
	public $user;
	public $pass;
	public $sessid;
	public $groupsessid;
	public $grouplist;

	public $sException;

	// default constructor
	public function __construct()
	{
	}
	
	// Function call
	public function FunctionCall($funcname, $param1 = "", $param2 = "", $param3 = "", $param4 = ""){
		@$result = icewarp_calendarfunctioncall($funcname, $param1, $param2, $param3, $param4);
		
		return $result;
	}

	public function CreateParamLine($array){
		if(!is_array($array)) return '';
		$result = '';
		foreach($array as $k => $v) $result .= $k. "=" . rawurlencode($v) ."&";
		return $result;
	}

	public function ParseParamLine($line){
		$result = array();
		
		if (!$line) return array();

		$lines = explode("\r\n", $line);

		// Get field array list
		$fields = explode("&", strtoupper(trim($lines[0])));
		unset($lines[0]);

		// Get all lines array
		foreach ($lines as $row){
		$row = trim($row);
		if (!$row) continue;
		$arow = explode("&", $row);

		foreach ($fields as $k => $field){
			if($field) $item[$field] = rawurldecode($arow[$k]);
		}
		$result[] = $item;
		}

		return $result;
	}

	// Session functions
	public function Login(){
		global $SERVER_NAME;

		if (!$this->user) {
		 $this->sException = 'No user';
		 return false;
		}
		if (!$this->pass) {
		 $this->sException = 'No pass';
		 return false;
		}

		if($this->IsConnected()) return $this->sessid;

		return $this->sessid = $this->FunctionCall("loginuser", $this->user, $this->pass, $SERVER_NAME);
	}

	public function Logout(){
		$this->FunctionCall("logoutuser", $this->sessid);

		$this->sessid = null;
		$this->grouplist = null;

	}

	public function IsConnected(){
		//API function is called once on first IsConnected call in one request
		if ($_SESSION['REQUEST_UID']==$this->requestUID) 
			return true;
		if ($this->sessid && $this->FunctionCall("userloggedon", $this->sessid)){
			$this->requestUID = $_SESSION['REQUEST_UID'];
			return true;
		}
		//If not connected , close group & folder access
		$this->cleanup();
		return false;
	}

	// Groups
	public function OpenGroup($groupid="*"){
    
    if (!$groupid) $groupid = $this->grouplist[0]["GRP_ID"];
    
    if($_SESSION['GW'][$this->user]['groups'][$groupid]==$_SESSION['GW'][$this->user]['groups']['*'])
      $groupid = '*';
	
		
		//Check if group is already opened and return its group sessid
		if (isset($_SESSION['GW'][$this->user]['groups'][$groupid])) {
			return $_SESSION['GW'][$this->user]['groups'][$groupid];
		}
		
 

		//Store this group as opened 
		$_SESSION['GW'][$this->user]['groups'][$groupid] = $this->FunctionCall("opengroup", $this->sessid,$groupid);

		
		return $_SESSION['GW'][$this->user]['groups'][$groupid];
	}

	public function CloseGroup($groupsessid){
		
		//Remove this group from opened groups
		$groupid = @array_search($groupsessid,$_SESSION['GW'][$this->user]['groups']);


  /* if ($groupid!='*'){*/
    unset($_SESSION['GW'][$this->user]['groups'][$groupid]);
    $result = $this->FunctionCall("closegroup", $groupsessid);
  /* }*/

        
		return $result;
		
	}

	public function OpenFolder($groupsessid, $folder){


		//Check if folder is already opened and return its groupsessid
		if (isset($_SESSION['GW'][$this->user]['folders'][$folder])) {
			return $_SESSION['GW'][$this->user]['folders'][$folder];
		}
		//Store this folder as opened 
    $_SESSION['GW'][$this->user]['folders'][$folder] = $this->FunctionCall("openfolder", $groupsessid, $folder);
    
		return $_SESSION['GW'][$this->user]['folders'][$folder];
	}

	public function CloseFolder($groupsessid){
	  //echo "Closing folder";
		$folderid = @array_search($groupsessid,$_SESSION['GW'][$this->user]['folders']);
		unset($_SESSION['GW'][$this->user]['folders'][$folderid]);
		
		return $this->FunctionCall('closefolder',$groupsessid);
	}

	public function GetFolders($groupsessid){
		return $this->ParseParamLine($this->FunctionCall("getfolderlist", $groupsessid));
	}

	public function GetGroups($groupview=""){
		if (!$this->sessid){
		 $this->sException = 'Call login first';
		 return false;
		}

		if(is_array($this->grouplist)) return $this->grouplist;

		return $this->grouplist = $this->ParseParamLine($this->FunctionCall("getgrouplist", $this->sessid,$groupview));
	}

	public function GetGroupSessid($groupid="*"){
		return $this->OpenGroup($groupid);
	}

	public function GetGroupRights($groupsessid){
		return $this->FunctionCall("GetGroupAccessRights",$groupsessid);
	}

  public function TimeZone(&$aItem,$inout = 'in')
  {
    
    $aItem = array_change_key_case($aItem,CASE_UPPER);
    //Event or contact

    if ($aItem['CNT_ID']) return;

    if (!isset($this->seconds)){
      $stz = $_SESSION['TIMEZONE'];

     if(isset($aItem['CTZ'])){
        $ctz = $aItem['CTZ'];
        $ctz = $ctz * 60;
      }else{
        return;
      }
      $this->seconds = $stz - $ctz;
    }
    unset($aItem['CTZ']);
   // dmp($aItem);
    if ($inout == 'out') $seconds = (-1)*$this->seconds;
    else $seconds = $this->seconds;

    if ($aItem['EVNSTARTDATE'] && $aItem['EVNSTARTTIME'] && intval($aItem['EVNSTARTTIME'])!=-1){
      $unix = self::calendar2unixTime($aItem['EVNSTARTDATE'],$aItem['EVNSTARTTIME']);
      $unix+= $seconds;
      $aItem['EVNSTARTTIME'] = self::unix2calendarTime($unix);
      $aItem['EVNSTARTDATE'] = self::unix2calendarDate($unix);

    }
    
    if ($aItem['EVNENDDATE'] && $aItem['EVNENDTIME'] && intval($aItem['EVNENDTIME'])!=-1){
      $unix = self::calendar2unixTime($aItem['EVNENDDATE'],$aItem['EVNENDTIME']);
      $unix+= $seconds;
      $aItem['EVNENDTIME'] = self::unix2calendarTime($unix);
      $aItem['EVNENDDATE'] = self::unix2calendarDate($unix);    
    }

  }
	// Date and time functions
	public static function unix2calendarDate($unixDate)
	{
/*	 // strip leading # if exists
	 if (substr($unixDate, 0, 1) === '#')
	    $unixDate = substr($unixDate, 1);*/
	 
	 $arr = getdate(intval($unixDate));
	 return GregorianToJD($arr["mon"], $arr["mday"], $arr["year"]);
	}
	public static function unix2calendarTime($unixDate) 
	{
/*		// if leading # exists, return -1 (indicates a whole day event)
		if (substr($unixDate, 0, 1) === '#')
	    return -1;*/
		
		$arr = getdate($unixDate);
		return $arr["hours"] * 60 + $arr["minutes"];
	}
	public static function calendar2unixTime($calendarDate, $calendarTime) 
	{
		$date = JDToGregorian(intval($calendarDate));

		ereg("([0-9]{1,2})/([0-9]{1,2})/([0-9]{4})", $date, $regs);
		$month = $regs[1];
		$day = $regs[2];
		$year = $regs[3];
		if (intval($calendarTime) === -1) {
			$hour = 0;
			$min = 0;
			$noTime = true;
		} else {
			$hour = floor($calendarTime / 60);
			$min = $calendarTime % 60;
			$noTime = false;
		}
		
		$result = mktime($hour, $min, 0, $month, $day, $year);
		
		// if not defined, return empty string
		if ($result === false || $result === -1)
			$result = '';
/*		// if no time is defined, append # before the unixTime
		else if ($noTime) $result = '#' . $result;*/
			
		return $result;
	}

	public function __destruct()
	{
  //		$this->Logout();
	}
	
	// TODO
	// clean all session records made up by this class
	public function cleanup()
	{
	   if (isset($_SESSION['GW'])) foreach($_SESSION['GW'] as $gwaccount)
	   {
	       if (isset($gwaccount['folders'])) foreach($gwaccount['folders'] as $folder){

               $this->CloseFolder($folder);

               
         }
	   
	       if (isset($gwaccount['groups'])) foreach($gwaccount['groups'] as $key => $group)
	       {
              
                $this->CloseGroup($group);  
     
        }
     }           
	   unset($_SESSION['GW']);
	}
}

?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/server/inc/gw/gwapi.php <==
<?
class MerakGWAPI {

	// PUBLIC
	public $user;
	public $pass;
	public $sessid;
	public $groupsessid;
	public $grouplist;

	public $sException;

	// default constructor
	public function __construct()
	{
	}
	
	// Function call
	public function FunctionCall($funcname, $param1 = "", $param2 = "", $param3 = "", $param4 = ""){
		@$result = icewarp_calendarfunctioncall($funcname, $param1, $param2, $param3, $param4);
		
		return $result;
	}

	public function CreateParamLine($array){
		if(!is_array($array)) return '';
		$result = '';
		foreach($array as $k => $v) $result .= $k. "=" . rawurlencode($v) ."&";
		return $result;
	}

	public function ParseParamLine($line){
		$result = array();
		
		if (!$line) return array();

		$lines = explode("\r\n", $line);

		// Get field array list
		$fields = explode("&", strtoupper(trim($lines[0])));
		unset($lines[0]);

		// Get all lines array
		foreach ($lines as $row){
		$row = trim($row);
		if (!$row) continue;
		$arow = explode("&", $row);

		foreach ($fields as $k => $field){
			if($field) $item[$field] = rawurldecode($arow[$k]);
		}
		$result[] = $item;
		}

		return $result;
	}

	// Session functions
	public function Login(){
		global $SERVER_NAME;

		if (!isset($this->user)) {
		 $this->sException = 'No user';
		 return false;
		}
		if (!isset($this->pass)) {
		 $this->sException = 'No pass';
		 return false;
		}

		if($this->IsConnected()) return $this->sessid;

		return $this->sessid = $this->FunctionCall("loginuser", $this->user, $this->pass, $SERVER_NAME);
		//return false;
	}

	public function Logout(){
		$this->FunctionCall("logoutuser", $this->sessid);

		$this->sessid = null;
		$this->grouplist = null;

	}

	public function IsConnected(){
		//API function is called once on first IsConnected call in one request
		if ($_SESSION['REQUEST_UID']==$this->requestUID && $_SESSION['REQUEST_UID']!='') 
			return true;
			
		if ($this->sessid && $this->FunctionCall("userloggedon", $this->sessid)){
			$this->requestUID = $_SESSION['REQUEST_UID'];
			return true;
		}
		//If not connected , close group & folder access
		$this->cleanup();
		return false;
	}

	// Groups
	public function OpenGroup($groupid="*"){
    
    if (!$groupid) $groupid = $this->grouplist[0]["GRP_ID"];
    
    if($_SESSION['GW'][$this->user]['groups'][$groupid]==$_SESSION['GW'][$this->user]['groups']['*'])
      $groupid = '*';
	
		
		//Check if group is already opened and return its group sessid
		if (isset($_SESSION['GW'][$this->user]['groups'][$groupid])) {
			return $_SESSION['GW'][$this->user]['groups'][$groupid];
		}
		
 

		//Store this group as opened 
		$_SESSION['GW'][$this->user]['groups'][$groupid] = $this->FunctionCall("opengroup", $this->sessid,$groupid);

		
		return $_SESSION['GW'][$this->user]['groups'][$groupid];
	}

	public function CloseGroup($groupsessid){
		
		//Remove this group from opened groups
		$groupid = @array_search($groupsessid,$_SESSION['GW'][$this->user]['groups']);


  /* if ($groupid!='*'){*/
    unset($_SESSION['GW'][$this->user]['groups'][$groupid]);
    $result = $this->FunctionCall("closegroup", $groupsessid);
  /* }*/

        
		return $result;
		
	}

	public function OpenFolder($groupsessid, $folder){


		//Check if folder is already opened and return its groupsessid
		if (isset($_SESSION['GW'][$this->user]['folders'][$folder])) {
			return $_SESSION['GW'][$this->user]['folders'][$folder];
		}
		//Store this folder as opened 
    $_SESSION['GW'][$this->user]['folders'][$folder] = $this->FunctionCall("openfolder", $groupsessid, $folder);
    
		return $_SESSION['GW'][$this->user]['folders'][$folder];
	}

	public function CloseFolder($groupsessid){
		$folderid = @array_search($groupsessid,$_SESSION['GW'][$this->user]['folders']);
		unset($_SESSION['GW'][$this->user]['folders'][$folderid]);
		
		return $this->FunctionCall('closefolder',$groupsessid);
	}

	public function GetFolders($groupsessid){
		return $this->ParseParamLine($this->FunctionCall("getfolderlist", $groupsessid));
	}

	public function GetGroups($groupview=""){
		if (!$this->sessid){
		 $this->sException = 'Call login first';
		 return false;
		}

		if(is_array($this->grouplist)) return $this->grouplist;

		return $this->grouplist = $this->ParseParamLine($this->FunctionCall("getgrouplist", $this->sessid,$groupview));
	}

	public function GetGroupSessid($groupid="*"){
		return $this->OpenGroup($groupid);
	}

	public function GetGroupRights($groupsessid){
		return $this->FunctionCall("GetGroupAccessRights",$groupsessid);
	}

  public function TimeZone(&$aItem,$inout = 'in')
  {
    
    $aItem = @array_change_key_case($aItem,CASE_UPPER);
    //Event or contact

    if ($aItem['CNT_ID']) return;

    if (!isset($this->seconds)){
      $stz = $_SESSION['TIMEZONE'];

     if(isset($aItem['CTZ'])){
        $ctz = $aItem['CTZ'];
        $ctz = $ctz * 60;
      }else{
        return;
      }
      $this->seconds = $stz - $ctz;
    }
    unset($aItem['CTZ']);

    if ($inout == 'out') $seconds = (-1)*$this->seconds;
    else $seconds = $this->seconds;

    if ($aItem['EVNSTARTDATE'] && $aItem['EVNSTARTTIME'] && intval($aItem['EVNSTARTTIME'])!=-1){
      $unix = self::calendar2unixTime($aItem['EVNSTARTDATE'],$aItem['EVNSTARTTIME']);
      $unix+= $seconds;
      $aItem['EVNSTARTTIME'] = self::unix2calendarTime($unix);
      $aItem['EVNSTARTDATE'] = self::unix2calendarDate($unix);

    }
    
    if ($aItem['EVNENDDATE'] && isset($aItem['EVNENDTIME']) && intval($aItem['EVNENDTIME'])!=-1){
      $unix = self::calendar2unixTime($aItem['EVNENDDATE'],$aItem['EVNENDTIME']);
      $unix+= $seconds;

      $aItem['EVNENDTIME'] = self::unix2calendarTime($unix);
      $aItem['EVNENDDATE'] = self::unix2calendarDate($unix);
    }

  }
	// Date and time functions
	public static function unix2calendarDate($unixDate)
	{
/*	 // strip leading # if exists
	 if (substr($unixDate, 0, 1) === '#')
	    $unixDate = substr($unixDate, 1);*/
	 
	 $arr = getdate(intval($unixDate));
	 return GregorianToJD($arr["mon"], $arr["mday"], $arr["year"]);
	}
	public static function unix2calendarTime($unixDate) 
	{
/*		// if leading # exists, return -1 (indicates a whole day event)
		if (substr($unixDate, 0, 1) === '#')
	    return -1;*/
		
		$arr = getdate($unixDate);
  	return $arr["hours"] * 60 + $arr["minutes"];
	}
	public static function calendar2unixTime($calendarDate, $calendarTime) 
	{
    $date = JDToGregorian($calendarDate);
		ereg("([0-9]{1,2})/([0-9]{1,2})/([0-9]{4})", $date, $regs);
		$month = $regs[1];
		$day = $regs[2];
		$year = $regs[3];
		if (intval($calendarTime) === -1) {
			$hour = 0;
			$min = 0;
			$noTime = true;
		} else {
			$hour = floor($calendarTime / 60);
			$min = $calendarTime % 60;
			$noTime = false;
		}
		
		$result = mktime($hour, $min, 0, $month, $day, $year);
		
		// if not defined, return empty string
		if ($result === false || $result === -1)
			$result = '';
/*		// if no time is defined, append # before the unixTime
		else if ($noTime) $result = '#' . $result;*/
			//echo $result;
		return $result;
	}

	public function __destruct()
	{
  //		$this->Logout();
	}
	
	// TODO
	// clean all session records made up by this class
	public function cleanup()
	{
	   if (isset($_SESSION['GW'])) foreach($_SESSION['GW'] as $gwaccount)
	   {
	       if (isset($gwaccount['folders'])) foreach($gwaccount['folders'] as $folder){

               $this->CloseFolder($folder);

               
         }
	   
	       if (isset($gwaccount['groups'])) foreach($gwaccount['groups'] as $key => $group)
	       {
              
                $this->CloseGroup($group);  
     
        }
     }           
	   unset($_SESSION['GW']);
	}
}

?>
